#    _/        _/_/_/_/  _/_/_/            _/_/_/    _/_/_/                  _/_/_/  _/_/_/_/_/    _/_/    _/_/_/_/_/  _/    _/    _/_/_/   
#   _/        _/        _/    _/          _/    _/    _/                  _/            _/      _/    _/      _/      _/    _/  _/          
#  _/        _/_/_/    _/    _/          _/_/_/      _/  _/_/_/_/_/        _/_/        _/      _/_/_/_/      _/      _/    _/    _/_/       
# _/        _/        _/    _/          _/          _/                        _/      _/      _/    _/      _/      _/    _/        _/      
#_/_/_/_/  _/_/_/_/  _/_/_/            _/        _/_/_/                _/_/_/        _/      _/    _/      _/        _/_/    _/_/_/         
#                                                                                                                                          
#!/usr/bin/env python3
#
# Shows your raspberry pi status (RAM/DISK/TEMP)* on a ws281 RGB LED Strip
# 
# This program consists of a mode selection button which is not necessary!
#
# button GPIO: 1 and 11
# Strip LED GPIO: 10-data, GND and +Ve can 
# either be connected to an external source or can also 
# be used directly using the 5V or 3.3V pins in raspberry pi.
#
# You can simply edit the default mode num 4, ie; 4th elif statement and give your
# desired mode so that you do not have to use the button.
#
# you can also set your own time periods for each display by changing the
# values given in wait_ms(in Milliseconds)
#
#*You can also add more feautures(ie; cpu usage, tasks count etc.) into the program through psutil
#
#
#-------------------------------------------------------------------------Written By Anand Krishnan
                                                                                                        
import RPi.GPIO as GPIO
import time 
from rpi_ws281x import * 
import argparse
import psutil

GPIO.setmode(GPIO.BOARD)
GPIO.setup(11,GPIO.IN, pull_up_down=GPIO.PUD_DOWN)



          #MAIN CONFIGURATIONS:
LED_COUNT      = 8       #number of LED pixels.
LED_PIN        = 18      #GPIO pin connected to the pixels (18 uses PWM!).
#LED_PIN        = 10      #GPIO pin connected to the pixels (10 uses SPI /dev/spidev0.0).
LED_FREQ_HZ    = 800000  #LED signal frequency in hertz (usually 800khz)
LED_DMA        = 10      #DMA channel to use for generating signal (try 10)
LED_BRIGHTNESS = 10      #set to 0 for darkest and 255 for brightest
LED_INVERT     = False   #True to invert the signal (when using NPN transistor level shift)
LED_CHANNEL    = 0       #set to '1' for GPIOs 13, 19, 41, 45 or 53

MAX_TEMP       = 70      #set Maximum Temperature out of which it should be displayed !NOT THE ALERT TEMP!
ALERT_VALUE    = 70      #set peak point to trigger the alert signal 
WHITE          = 1       #show the remaining led bar with white colour
MAX_MODES      = 6       #only change if you are adding more modes into the program 
                         #!! PLEASE NOTE THAT THIS PROGRAM WIL NOT BE ABLE TO SHOW/SELECT THE MODES IF MAX_MODES > LED_COUNT !!



# Define functions for showing various proccess/status
                    #READ VALUES FROM THE SYSTEM
def disk_percent():
    disk = psutil.disk_usage('/').percent
    return disk

def mem_percent():
    mem = psutil.virtual_memory().percent
    return mem
    
def temp_celcius():
    sensor = psutil.sensors_temperatures(fahrenheit=False)
	current_temp = sensor['cpu_thermal'][0].current
    return current_temp
 
			#TO WIPE THROUGH THE STRIP COMPLETELY (CAN ALSO BE USED TO SHOW SPECIFIC COLOUR ON THE COMPLETE LED BAR)
def colorWipe(strip, color, wait_ms=50):   
    for i in range(strip.numPixels()):
        strip.setPixelColor(i, color)
        strip.show()
        time.sleep(wait_ms/1000.0)

			 #TO GLOW AT A SINGLE POINT
def point(strip, i, color, wait_ms):    
    colorWipe(strip, Color(0,0,0), 10)
    strip.setPixelColor(i, color)
    strip.show()
    time.sleep(wait_ms/1000.0)     
    
              		 #LOADING GREEN
def load(strip, wait_ms=120):
    for x in range(1):
        for i in range(strip.numPixels()): #run forwards
            strip.setPixelColor(i-1, Color(0,45,0))
            strip.setPixelColor(i, Color(0,255,0))
            strip.setPixelColor(i+1, Color(0,45,0))
            strip.show()
            time.sleep(wait_ms/1000.0)
            strip.setPixelColor(i-1, Color(0,0,0))
            strip.setPixelColor(i, Color(0,0,0))
            strip.setPixelColor(i+1, Color(0,0,0))

        for i in range(strip.numPixels(),0,-1): #run backwards
            strip.setPixelColor(i-1, Color(0,45,0))
            strip.setPixelColor(i, Color(0,255,0))
            strip.setPixelColor(i+1, Color(0,45,0))
            strip.show()
            time.sleep(wait_ms/1000.0)
            strip.setPixelColor(i-1, Color(0,0,0))
            strip.setPixelColor(i, Color(0,0,0))
            strip.setPixelColor(i+1, Color(0,0,0))

    colorWipe(strip, Color(0,0,0), 10)
    

			#SHOWS THE MEMMORY USAGE            
def mem(strip, color, wait_ms=300):       
	mem = mem_percent()
	led_num = mem /(100/ strip.numPixels())
	Int_led_num = int(led_num)
	for i in range(strip.numPixels()):
		if i>(Int_led_num-1):
			wait_ms = 280
		if (i+1)>Int_led_num and WHITE:
			strip.setPixelColor(i, Color(250,250,250))
		elif (i+1)>Int_led_num:
 			break
		else:
			strip.setPixelColor(i, color)
		strip.show()
		time.sleep(wait_ms/1000.0)
	colorWipe(strip, Color(0,0,0), 10)

			#SHOWS THE DISK USAGE
def disk(strip, color, wait_ms=300):    
	disk = disk_percent()
	led_num = disk /(100/ strip.numPixels())
	Int_led_num = int(led_num)
	for i in range(strip.numPixels()):
		if (i+1)>Int_led_num and WHITE:
			strip.setPixelColor(i, Color(150,150,150))
		elif (i+1)>Int_led_num:
 			break
		else:
			strip.setPixelColor(i, color)
		strip.show()
		time.sleep(wait_ms/1000.0)
	colorWipe(strip, Color(0,0,0), 10)

			#SHOWS TEMPERATURE
def temp(strip, color, wait_ms=300):    
	current_temp = temp_celcius()
	led_num = current_temp /(MAX_TEMP/ strip.numPixels())
	Int_led_num = int(led_num)
	for i in range(strip.numPixels()):
		if (i+1)>Int_led_num and WHITE:
			strip.setPixelColor(i, Color(55,149,255))
		elif (i+1)>Int_led_num:
 			break
		else:
			strip.setPixelColor(i, color)
		strip.show()
		time.sleep(wait_ms/1000.0)
	colorWipe(strip, Color(0,0,0), 10)



			#CHANGE MODES FUNCTION: THIS CYCLES THROUGH THE AVAILABLE MODES
def modes(x):
    for i in range(2):
        load(strip)
        colorWipe(strip, Color(0,0,0), 10)
        if GPIO.event_detected(11):
            for x in range(20):
                if x == MAX_MODES:   #maximum number of modes  !you can change it in main config(above)!
                    point(strip, (x-1), Color(150,150,150),500)
                    return x
                elif GPIO.event_detected(11):
                    point(strip, (x-1), Color(150,150,150),500)
                    return x
                else:
                    point(strip, x, Color(0,0,250),1000)
            break
        else:
            if i == 1:
                point(strip, (x - 1), Color(0,0,250),500)
                point(strip, (x - 1), Color(120,120,120),500)
                time.sleep(0.5)
                return x

			#ALERT IF TEMP OR RAM IS MORE THAN ALERT_VALUE
def alert_check():            
    cpu_therm = psutil.sensors_temperatures(fahrenheit=False)
    current_temp = cpu_therm['cpu_thermal'][0].current #Temperature in Celcius
    mem_percent = psutil.virtual_memory().percent #memmory in percentage
    if mem_percent > ALERT_VALUE or current_temp > ALERT_VALUE:
        return 1


                        # MAIN PROGRAM
if __name__ == '__main__':
    # Process arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--clear', action='store_true', help='clear the display on exit')
    args = parser.parse_args()

    # Create NeoPixel object with appropriate configuration.
    strip = Adafruit_NeoPixel(LED_COUNT, LED_PIN, LED_FREQ_HZ, LED_DMA, LED_INVERT, LED_BRIGHTNESS, LED_CHANNEL)
    strip.begin()
    GPIO.add_event_detect(11,GPIO.RISING,bouncetime=700) # Setup event on pin 11 ;button detect
    mode = 4               #CHANGE THIS TO DESIRED MODE (TO CHANGE MODE WITHOUT A BUTTON)


    print ('\n BLUE: MEMMORY \n GREEN: DISK \n RED: TEMPERATURE IN CELCIUS \n Ctrl-C to Exit')  #COMMENT HERE WHEN YOU SET THIS ON YOUR STARTUP
    if not args.clear:
        print('Use "-c" argument to clear LEDs on exit')  #COMMENT HERE & UNCOMMENT PASS WHEN YOU SET THIS ON TO YOUR STARTUP
#        pass
    try:

        while True:
            if GPIO.event_detected(11): 
                mode = modes(mode)
            alert = alert_check()
            if alert == 1:                      #Alert mode to notify if your pi is too warm or using too much ram (DEFAULT:70 Celcius / 70 Percent)
               colorWipe(strip, Color(250,0,0), 50) 
               colorWipe(strip, Color(140,0,0), 50) 
            elif mode == 1:                     #you can add more elif statements to specify your own modes
                mem(strip, Color(0, 0, 255))
            elif mode == 2:
                disk(strip, Color(0, 255, 0))
            elif mode == 3:
                temp(strip, Color(255, 0, 0))
            elif mode == 4:                    #DEFAULT MODE
                disk(strip, Color(0, 255, 0))
                mem(strip, Color(0, 0, 255))
                temp(strip, Color(255, 0, 0))
                colorWipe(strip, Color(0,0,0), 10)
            elif mode == 5:                   #shows only if any of the three cross 50%
                if disk_percent() > 50:
                disk(strip, Color(0, 255, 0))
                colorWipe(strip, Color(0,0,0), 10)
                elif mem_percent() > 50:
                mem(strip, Color(0, 0, 255))
                colorWipe(strip, Color(0,0,0), 10)
                elif temp_celcius() > 50:
                temp(strip, Color(255, 0, 0))
                colorWipe(strip, Color(0,0,0), 10)
            elif mode == 6:                    #Alert mode only
                colorWipe(strip, Color(0,0,0), 10)

    except KeyboardInterrupt:
        if args.clear:
            colorWipe(strip, Color(0,0,0), 10)
            GPIO.cleanup()
 